@@ -18,14 +18,13 @@
 
 package org.apache.flink.ml.common.datastream;
 
+import org.apache.flink.api.common.functions.AggregateFunction;
 import org.apache.flink.api.common.functions.MapPartitionFunction;
 import org.apache.flink.api.common.functions.ReduceFunction;
 import org.apache.flink.api.common.functions.RichMapPartitionFunction;
-import org.apache.flink.api.common.restartstrategy.RestartStrategies;
 import org.apache.flink.api.common.typeinfo.Types;
-import org.apache.flink.configuration.Configuration;
+import org.apache.flink.ml.util.TestUtils;
 import org.apache.flink.streaming.api.datastream.DataStream;
-import org.apache.flink.streaming.api.environment.ExecutionCheckpointingOptions;
 import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
 import org.apache.flink.util.Collector;
 import org.apache.flink.util.NumberSequenceIterator;
@@ -46,12 +45,7 @@ public class DataStreamUtilsTest {
 
     @Before
     public void before() {
-        Configuration config = new Configuration();
-        config.set(ExecutionCheckpointingOptions.ENABLE_CHECKPOINTS_AFTER_TASKS_FINISH, true);
-        env = StreamExecutionEnvironment.getExecutionEnvironment(config);
-        env.setParallelism(4);
-        env.enableCheckpointing(100);
-        env.setRestartStrategy(RestartStrategies.noRestart());
+        env = TestUtils.getExecutionEnvironment();
     }
 
     @Test
@@ -75,6 +69,52 @@ public class DataStreamUtilsTest {
         assertArrayEquals(new long[] {190L}, sum.stream().mapToLong(Long::longValue).toArray());
     }
 
+    @Test
+    public void testAggregate() throws Exception {
+        DataStream<Long> dataStream =
+                env.fromParallelCollection(new NumberSequenceIterator(0L, 19L), Types.LONG);
+        DataStream<String> result = DataStreamUtils.aggregate(dataStream, new TestAggregateFunc());
+        List<String> stringSum = IteratorUtils.toList(result.executeAndCollect());
+        assertEquals(1, stringSum.size());
+        assertEquals("190", stringSum.get(0));
+    }
+
+    @Test
+    public void testAggregateWithNonNeutralInitialAccumulator() throws Exception {
+        DataStream<Long> dataStream =
+                env.fromParallelCollection(new NumberSequenceIterator(0L, 19L), Types.LONG);
+        DataStream<String> result =
+                DataStreamUtils.aggregate(
+                        dataStream, new TestAggregateFuncWithNonNeutralInitialAccumulator());
+        List<String> stringSum = IteratorUtils.toList(result.executeAndCollect());
+        assertEquals(1, stringSum.size());
+        assertEquals(Integer.toString(190 + env.getParallelism()), stringSum.get(0));
+
+        env.setParallelism(env.getParallelism() + 1);
+        dataStream = env.fromParallelCollection(new NumberSequenceIterator(0L, 19L), Types.LONG);
+        result =
+                DataStreamUtils.aggregate(
+                        dataStream, new TestAggregateFuncWithNonNeutralInitialAccumulator());
+        stringSum = IteratorUtils.toList(result.executeAndCollect());
+        assertEquals(1, stringSum.size());
+        assertEquals(Integer.toString(190 + env.getParallelism()), stringSum.get(0));
+    }
+
+    @Test
+    public void testSample() throws Exception {
+        int numSamples = 10;
+        int[] totalMinusOneChoices = new int[] {0, 5, 9, 10, 11, 20, 30, 40, 200};
+        for (int totalMinusOne : totalMinusOneChoices) {
+            DataStream<Long> dataStream =
+                    env.fromParallelCollection(
+                            new NumberSequenceIterator(0L, totalMinusOne), Types.LONG);
+            DataStream<Long> result = DataStreamUtils.sample(dataStream, numSamples, 0);
+            //noinspection unchecked
+            List<String> sampled = IteratorUtils.toList(result.executeAndCollect());
+            assertEquals(Math.min(numSamples, totalMinusOne + 1), sampled.size());
+        }
+    }
+
     @Test
     public void testGenerateBatchData() throws Exception {
         DataStream<Long> dataStream =
@@ -99,4 +139,38 @@ public class DataStreamUtilsTest {
             out.collect(cnt);
         }
     }
+
+    /** A simple implementation for {@link AggregateFunction}. */
+    private static class TestAggregateFunc implements AggregateFunction<Long, Long, String> {
+        @Override
+        public Long createAccumulator() {
+            return 0L;
+        }
+
+        @Override
+        public Long add(Long element, Long acc) {
+            return element + acc;
+        }
+
+        @Override
+        public String getResult(Long acc) {
+            return String.valueOf(acc);
+        }
+
+        @Override
+        public Long merge(Long acc1, Long acc2) {
+            return acc1 + acc2;
+        }
+    }
+
+    /**
+     * An extension for {@link TestAggregateFunc} that provides a non-neutral initial accumulator.
+     */
+    private static class TestAggregateFuncWithNonNeutralInitialAccumulator
+            extends TestAggregateFunc {
+        @Override
+        public Long createAccumulator() {
+            return 1L;
+        }
+    }
 }
