@@ -21,6 +21,7 @@ package org.apache.flink.ml.classification.naivebayes;
 import org.apache.flink.api.common.functions.MapFunction;
 import org.apache.flink.api.common.serialization.Encoder;
 import org.apache.flink.api.common.typeinfo.TypeInformation;
+import org.apache.flink.api.common.typeinfo.Types;
 import org.apache.flink.api.common.typeutils.base.DoubleSerializer;
 import org.apache.flink.api.common.typeutils.base.MapSerializer;
 import org.apache.flink.configuration.Configuration;
@@ -31,6 +32,7 @@ import org.apache.flink.core.memory.DataOutputViewStreamWrapper;
 import org.apache.flink.ml.linalg.DenseVector;
 import org.apache.flink.ml.linalg.Vector;
 import org.apache.flink.ml.linalg.typeinfo.DenseVectorSerializer;
+import org.apache.flink.ml.linalg.typeinfo.DenseVectorTypeInfo;
 import org.apache.flink.streaming.api.datastream.DataStream;
 import org.apache.flink.table.api.Table;
 import org.apache.flink.table.api.bridge.java.StreamTableEnvironment;
@@ -50,6 +52,21 @@ import java.util.Map;
  * to save/load model data.
  */
 public class NaiveBayesModelData {
+
+    private static final Map<String, TypeInformation<?>> fields;
+
+    static {
+        fields = new HashMap<>();
+        fields.put(
+                "theta",
+                Types.OBJECT_ARRAY(Types.OBJECT_ARRAY(Types.MAP(Types.DOUBLE, Types.DOUBLE))));
+        fields.put("piArray", DenseVectorTypeInfo.INSTANCE);
+        fields.put("labels", DenseVectorTypeInfo.INSTANCE);
+    }
+
+    public static final TypeInformation<NaiveBayesModelData> TYPE_INFO =
+            Types.POJO(NaiveBayesModelData.class, fields);
+
     /**
      * Log of class conditional probabilities, whose dimension is C (number of classes) by D (number
      * of features).
@@ -87,7 +104,8 @@ public class NaiveBayesModelData {
                                         new NaiveBayesModelData(
                                                 (Map<Double, Double>[][]) row.getField(0),
                                                 ((Vector) row.getField(1)).toDense(),
-                                                ((Vector) row.getField(2)).toDense()));
+                                                ((Vector) row.getField(2)).toDense()),
+                        TYPE_INFO);
     }
 
     /** Data encoder for the {@link NaiveBayesModelData}. */
@@ -161,7 +179,7 @@ public class NaiveBayesModelData {
 
         @Override
         public TypeInformation<NaiveBayesModelData> getProducedType() {
-            return TypeInformation.of(NaiveBayesModelData.class);
+            return TYPE_INFO;
         }
     }
 }
