@@ -20,9 +20,9 @@ package org.apache.flink.ml.clustering;
 
 import org.apache.flink.api.common.JobID;
 import org.apache.flink.api.common.JobSubmissionResult;
-import org.apache.flink.api.common.restartstrategy.RestartStrategies;
 import org.apache.flink.api.common.typeinfo.TypeInformation;
 import org.apache.flink.configuration.Configuration;
+import org.apache.flink.configuration.CoreOptions;
 import org.apache.flink.configuration.RestOptions;
 import org.apache.flink.metrics.Gauge;
 import org.apache.flink.ml.clustering.kmeans.KMeans;
@@ -38,11 +38,11 @@ import org.apache.flink.ml.linalg.typeinfo.DenseVectorTypeInfo;
 import org.apache.flink.ml.util.InMemorySinkFunction;
 import org.apache.flink.ml.util.InMemorySourceFunction;
 import org.apache.flink.ml.util.TestUtils;
+import org.apache.flink.runtime.client.JobStatusMessage;
 import org.apache.flink.runtime.jobgraph.JobGraph;
 import org.apache.flink.runtime.minicluster.MiniCluster;
 import org.apache.flink.runtime.minicluster.MiniClusterConfiguration;
 import org.apache.flink.runtime.testutils.InMemoryReporter;
-import org.apache.flink.streaming.api.environment.ExecutionCheckpointingOptions;
 import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
 import org.apache.flink.table.api.Table;
 import org.apache.flink.table.api.bridge.java.StreamTableEnvironment;
@@ -51,8 +51,10 @@ import org.apache.flink.util.TestLogger;
 
 import org.apache.commons.collections.CollectionUtils;
 import org.junit.After;
+import org.junit.AfterClass;
 import org.junit.Assert;
 import org.junit.Before;
+import org.junit.BeforeClass;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.TemporaryFolder;
@@ -136,29 +138,20 @@ public class OnlineKMeansTest extends TestLogger {
     private InMemorySinkFunction<Row> outputSink;
     private InMemorySinkFunction<KMeansModelData> modelDataSink;
 
-    // TODO: creates static mini cluster once for whole test class after dependency upgrades to
-    // Flink 1.15.
-    private InMemoryReporter reporter;
-    private MiniCluster miniCluster;
-    private StreamExecutionEnvironment env;
-    private StreamTableEnvironment tEnv;
+    private static InMemoryReporter reporter;
+    private static MiniCluster miniCluster;
+    private static StreamExecutionEnvironment env;
+    private static StreamTableEnvironment tEnv;
 
     private Table offlineTrainTable;
     private Table onlineTrainTable;
     private Table onlinePredictTable;
 
-    @Before
-    public void before() throws Exception {
-        currentModelDataVersion = 0;
-
-        trainSource = new InMemorySourceFunction<>();
-        predictSource = new InMemorySourceFunction<>();
-        outputSink = new InMemorySinkFunction<>();
-        modelDataSink = new InMemorySinkFunction<>();
-
+    @BeforeClass
+    public static void beforeClass() throws Exception {
         Configuration config = new Configuration();
         config.set(RestOptions.BIND_PORT, "18081-19091");
-        config.set(ExecutionCheckpointingOptions.ENABLE_CHECKPOINTS_AFTER_TASKS_FINISH, true);
+        config.set(CoreOptions.DEFAULT_PARALLELISM, defaultParallelism);
         reporter = InMemoryReporter.create();
         reporter.addToConfiguration(config);
 
@@ -169,13 +162,20 @@ public class OnlineKMeansTest extends TestLogger {
                                 .setNumTaskManagers(numTaskManagers)
                                 .setNumSlotsPerTaskManager(numSlotsPerTaskManager)
                                 .build());
-        miniCluster.start();
 
-        env = StreamExecutionEnvironment.getExecutionEnvironment(config);
-        env.setParallelism(defaultParallelism);
-        env.enableCheckpointing(100);
-        env.setRestartStrategy(RestartStrategies.noRestart());
+        miniCluster.start();
+        env = TestUtils.getExecutionEnvironment(config);
         tEnv = StreamTableEnvironment.create(env);
+    }
+
+    @Before
+    public void before() throws Exception {
+        currentModelDataVersion = 0;
+
+        trainSource = new InMemorySourceFunction<>();
+        predictSource = new InMemorySourceFunction<>();
+        outputSink = new InMemorySinkFunction<>();
+        modelDataSink = new InMemorySinkFunction<>();
 
         offlineTrainTable = tEnv.fromDataStream(env.fromElements(trainData1)).as("features");
         onlineTrainTable =
@@ -188,6 +188,13 @@ public class OnlineKMeansTest extends TestLogger {
 
     @After
     public void after() throws Exception {
+        for (JobStatusMessage message : miniCluster.listJobs().get()) {
+            miniCluster.cancelJob(message.getJobId());
+        }
+    }
+
+    @AfterClass
+    public static void afterClass() throws Exception {
         miniCluster.close();
     }
 
